from petisco import RabbitMqConnector
from petisco.event.configurer.infrastructure.rabbitmq_event_configurer import (
    RabbitMqEventConfigurer,
)
from tests.modules.event.mothers.defaults import DEFAULT_ORGANIZATION, DEFAULT_SERVICE


class RabbitMqEventConfigurerMother:
    @staticmethod
    def default(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE)

    @staticmethod
    def with_retry_ttl_10ms(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE, retry_ttl=10
        )

    @staticmethod
    def with_main_and_retry_ttl_10ms(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE, retry_ttl=10, main_ttl=10
        )

    @staticmethod
    def with_service(service: str, connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector, DEFAULT_ORGANIZATION, service, retry_ttl=10
        )

    @staticmethod
    def with_ttl_1s(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE, retry_ttl=1000
        )



from typing import Dict

from meiga import Result, Error, Success

from tests.end2end.flask_app.sum.domain.interface_sum_executor import ISumExecutor


class SumExecutor(ISumExecutor):
    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def execute(self, value1: int, value2: int) -> Result[int, Error]:
        return Success(value1 + value2)



from petisco import Event
from petisco.event.legacy.subscriber.domain.subscriber_handler import subscriber_handler


@subscriber_handler()
def send_notification(event: Event):
    print(f"Event: {event}")



import inspect
import os
import time
from os import environ
from typing import Callable, Dict, Any

from dataclasses import dataclass

from petisco.event.bus.infrastructure.not_implemented_event_bus import (
    NotImplementedEventBus,
)
from petisco.event.configurer.infrastructure.not_implemented_configurer import (
    NotImplementedEventConfigurer,
)
from petisco.event.consumer.infrastructure.not_implemented_event_comsumer import (
    NotImplementedEventConsumer,
)
from petisco.event.legacy.publisher.domain.interface_event_publisher import (
    IEventPublisher,
)
from petisco.event.shared.domain.config_events import ConfigEvents
from petisco.event.shared.domain.service_deployed import ServiceDeployed
from petisco.event.legacy.subscriber.domain.interface_event_subscriber import (
    IEventSubscriber,
)
from petisco.event.shared.infrastructure.configure_events_infrastructure import (
    configure_events_infrastructure,
)
from petisco.frameworks.interface_application import IApplication
from petisco.logger.interface_logger import INFO, ILogger
from petisco.logger.log_message import LogMessage
from petisco.logger.not_implemented_logger import NotImplementedLogger
from petisco.notifier.infrastructure.not_implemented_notifier import (
    NotImplementedNotifier,
)
from petisco.notifier.domain.interface_notifier import INotifier
from petisco.notifier.domain.notifier_message import NotifierMessage
from petisco.application.config.config import Config
from petisco.application.singleton import Singleton
from petisco.application.interface_repository import IRepository
from petisco.application.interface_service import IService
from petisco.tasks.infrastructure.apscheduler_task_executor import (
    APSchedulerTaskExecutor,
)
from petisco import __version__
from petisco.tasks.infrastructure.not_implemented_task_executor import (
    NotImplementedTaskExecutor,
)


@dataclass
class Petisco(metaclass=Singleton):
    _app_name: str
    _app_version: str
    _logger: ILogger = NotImplementedLogger()
    _notifier: INotifier = NotImplementedNotifier()
    application: IApplication = None
    services_provider: Callable = None
    repositories_provider: Callable = None
    _options: Dict[str, Any] = None
    info: Dict = None
    _persistence_models: Dict[str, Dict[str, Any]] = None
    persistence_sources: Dict[str, Dict] = None
    config: Config = None
    event_publisher: IEventPublisher = None
    event_subscriber: IEventSubscriber = None
    _environment: str = None
    repositories: Dict[str, Any] = None
    services: Dict[str, Any] = None

    def __init__(self, config: Config):
        self.config = config
        self._app_name = config.app_name
        self._app_version = config.app_version
        self._logger = config.get_logger()
        self._environment = environ.get("ENVIRONMENT", None)
        self.info = {
            "app_name": self._app_name,
            "app_version": self._app_version,
            "petisco_version": __version__,
            "environment": self._environment,
            "elapsed_time": {},
        }
        self._notifier = config.get_notifier()
        self._set_persistence()
        self._set_events_configuration()
        self._set_tasks()
        self._options = config.options

    @staticmethod
    def get_instance():
        try:
            return Petisco()
        except Exception as e:  # noqa E722
            frame_info = inspect.stack()[1]
            raise ImportError(
                f"Petisco must be configured when the application starts. \n"
                f"Petisco has been requested before its initial configuration.\n "
                f"Following code must be executed after Petisco initialization:\n"
                f"\tfilename: {frame_info.filename}\n"
                f"\tlineno: {frame_info.lineno}\n"
                f"\tfunction: {frame_info.function}\n"
                f"\tcode_context: {frame_info.code_context}\n\n"
                f"\t Possible Solutions:\n"
                f"\t  * If you are testing and need any preconfigured Petisco, you can use the fixture given_any_petisco.\n"
                f"\t  * If you are testing and need specific preconfigured Petisco, you can create a fixture loading a petisco and then clearing.\n"
                f"\t  * If you are running a Petisco application, please review your loading process, petisco.yml is probably failing.\n"
            )

    @staticmethod
    def from_filename(filename: str):
        """
       Parameters
       ----------
       filename
           YAML-based configuration file (default petisco.yml)
        """
        print(f"Loading petisco from: {filename}")

        config = Config.from_filename(filename).unwrap_or_throw()

        return Petisco(config=config)

    def configure_events(self, filename: str):
        """
        Parameters
        ----------
        filename
            YAML-based event management configuration file (default petisco.events.yml)
        """

        message_broker_type = os.environ.get("PETISCO_EVENT_MESSAGE_BROKER")
        if message_broker_type and message_broker_type == "notimplemented":
            return

        config_events = ConfigEvents.from_filename(filename).unwrap_or_throw()
        self.event_bus, self.event_configurer, self.event_consumer = configure_events_infrastructure(
            config_events, self._logger
        )

        if config_events.event_subscribers:
            self.event_configurer.configure_subscribers(config_events.event_subscribers)
            self.event_consumer.add_subscribers(config_events.event_subscribers)
        else:
            self.event_configurer.configure()

        if config_events.store_queue_subscriber:
            self.event_consumer.add_handler_on_store(
                config_events.store_queue_subscriber
            )

        if config_events.queues_subscribers:
            for queue, handler in config_events.queues_subscribers.items():
                self.event_consumer.add_handler_on_queue(queue, handler)

        self.config_events = config_events

        self.info["config_events"] = self.config_events.info()

    def _publish_deploy_event(self):
        event = ServiceDeployed(app_name=self._app_name, app_version=self._app_version)

        if self.config_events and self.config_events.publish_deploy_event:
            self.event_bus.publish(event)

        # Legacy Approach
        if self.config.config_events.publish_deploy_event:
            self.event_publisher.publish(event)

    def _notify_deploy(self):
        self._notifier.publish(
            NotifierMessage(
                title="Service deployed",
                message=f"{self._app_name} has been deployed",
                info_petisco=self.get_info(),
            )
        )

    def _set_tasks(self):
        config_tasks = self.config.config_tasks
        if config_tasks and config_tasks.tasks:
            self.info["tasks"] = {}
            for task_name, config_task in config_tasks.tasks.items():
                self.info["tasks"][task_name] = config_task.to_dict()
            self.task_executor = APSchedulerTaskExecutor()
        else:
            self.task_executor = NotImplementedTaskExecutor()

    def _schedule_tasks(self):
        config_tasks = self.config.config_tasks
        if config_tasks.tasks:
            self.task_executor.start(config_tasks)

    def _unschedule_tasks(self):
        config_tasks = self.config.config_tasks
        if config_tasks.tasks:
            self.task_executor.stop()

    def _set_persistence(self):
        self._persistence_models = {}
        self.persistence_sources = {}
        config_persistence = self.config.config_persistence
        if config_persistence and config_persistence.configs:
            for config_key, config_value in config_persistence.configs.items():
                if config_value.type == "sql":
                    import_database_models_func = config_persistence.get_import_database_models_func(
                        config_key
                    )

                    config_value.config(import_database_models_func)

                    self._persistence_models[
                        config_key
                    ] = config_persistence.get_models(config_key)
                else:
                    config_value.config()
                self.persistence_sources[config_key] = {
                    "configured": True,
                    "type": config_value.type,
                }

    def _set_services_and_repositories_from_providers(self):
        config_providers = self.config.config_providers
        if not config_providers:
            return
        if config_providers.config_dependencies:
            config_providers.config_dependencies()

        if config_providers.services_provider:
            self.services = self._load_services(config_providers.services_provider)

            info_services = {}
            for key, service in self.services.items():
                if hasattr(service, "info"):
                    info_services[key] = service.info()
                else:
                    raise TypeError(
                        f"Service with key {key} ({type(service)}) must implement info"
                    )
            self.info["services"] = info_services
        if config_providers.repositories_provider:
            self.repositories = self._load_repositories(
                config_providers.repositories_provider
            )
            info_repositories = {}
            for key, repository in self.repositories.items():
                if hasattr(repository, "info"):
                    info_repositories[key] = repository.info()
                else:
                    raise TypeError(
                        f"Repository with key {key} ({type(repository)}) must implement info"
                    )

            self.info["repositories"] = info_repositories

    def _set_events_configuration(self):
        # New Approach
        self.event_bus = NotImplementedEventBus()
        self.event_configurer = NotImplementedEventConfigurer()
        self.event_consumer = NotImplementedEventConsumer()
        self.config_events = None

        # Legacy Approach
        config_events = self.config.config_events
        if not config_events:
            return

        if config_events.config_event_publisher.provider:
            self.event_publisher = config_events.config_event_publisher.provider()
            if hasattr(self.event_publisher, "info"):
                self.info["event_publisher"] = self.event_publisher.info()
            else:
                raise TypeError(
                    f"Given event_publisher ({type(self.event_publisher)}) must implement info"
                )

        if config_events.config_event_subscriber.provider:
            subscribers = config_events.config_event_subscriber.subscribers
            self.event_subscriber = config_events.config_event_subscriber.provider(
                subscribers
            )
            if hasattr(self.event_subscriber, "info"):
                self.info["event_subscriber"] = self.event_subscriber.info()
            else:
                raise TypeError(
                    f"Given event_subscriber ({type(self.event_subscriber)}) must implement info"
                )

    def _log_status(self):
        if self.info:
            self._logger.log(INFO, LogMessage(data={"message": {"info": self.info}}))

        if self._options:
            self._logger.log(
                INFO, LogMessage(data={"message": {"options": self._options}})
            )

    def _start(self):
        self._set_services_and_repositories_from_providers()
        self.event_subscriber.start()
        self.event_consumer.start()
        self._schedule_tasks()
        self._log_status()
        self._publish_deploy_event()
        self._notify_deploy()

    def load_services_and_repositories(self):
        self._set_services_and_repositories_from_providers()

    def start(self):
        self._start()
        self._application.start()

    def get_app(self):
        self._start()
        self._application = self.config.get_application()
        return self._application.get_app()

    def get_application(self):
        return self._application

    def stop(self):
        self.event_subscriber.stop()
        self.event_consumer.stop()
        self._unschedule_tasks()

    def _load_repositories(self, repositories_provider: Callable):
        start_time = time.time()
        repositories = repositories_provider()
        elapsed_time = time.time() - start_time
        self.info["elapsed_time"][
            "load_repositories"
        ] = f"{int(elapsed_time*1000.0)} ms"
        return repositories

    def _load_services(self, services_provider: Callable):
        start_time = time.time()
        services = services_provider()
        elapsed_time = time.time() - start_time
        self.info["elapsed_time"]["load_services"] = f"{int(elapsed_time * 1000.0)} ms"
        return services

    @staticmethod
    def get_service(key: str) -> IService:
        services = Petisco.get_instance().services
        if not services:
            raise ValueError(
                "Petisco: no service has been declared. Please, add it to petisco.yml."
            )
        service = services.get(key)
        if not service:
            raise ValueError(
                f"Petisco: {key} service is not defined. Please, add it to petisco.yml"
            )
        return service

    @staticmethod
    def get_repository(key: str) -> IRepository:
        repositories = Petisco.get_instance().repositories
        if repositories is None:
            raise ValueError(
                "Petisco: no repository has been declared. Please, add it to petisco.yml"
            )
        repository = repositories.get(key)
        if not repository:
            raise ValueError(
                f"Petisco: {key} repository is not defined. Please, add it to petisco.yml"
            )
        return repository

    @staticmethod
    def persistence_models(persistence_entry: str) -> Dict[str, str]:
        persistence_models = {}
        try:
            persistence_models = Petisco.get_instance()._persistence_models[
                persistence_entry
            ]
        except:  # noqa E722
            pass
        return persistence_models

    @staticmethod
    def get_persistence_model(persistence_entry: str, key: str) -> Any:
        return Petisco.persistence_models(persistence_entry).get(key)

    @staticmethod
    def persistence_session_scope():
        from petisco.persistence.sqlalchemy.sqlalchemy_session_scope import (
            session_scope,
        )

        return session_scope

    @staticmethod
    def get_event_bus():
        return Petisco.get_instance().event_bus

    @staticmethod
    def persistence_mongodb_database():
        from petisco.persistence.pymongo.pymongo_persistence import PyMongoPersistence

        return PyMongoPersistence().database

    @staticmethod
    def persistence_mongodb_client():
        from petisco.persistence.pymongo.pymongo_persistence import PyMongoPersistence

        return PyMongoPersistence().client

    @staticmethod
    def get_event_publisher():
        return Petisco.get_instance().event_publisher

    @staticmethod
    def get_event_subscriber():
        return Petisco.get_instance().event_subscriber

    @staticmethod
    def get_logger():
        return Petisco.get_instance()._logger

    @staticmethod
    def get_notifier():
        return Petisco.get_instance()._notifier

    @staticmethod
    def get_app_name():
        return Petisco.get_instance()._app_name

    @staticmethod
    def get_app_version():
        return Petisco.get_instance()._app_version

    @staticmethod
    def get_environment():
        return Petisco.get_instance()._environment

    @staticmethod
    def get_info():
        return Petisco.get_instance().info



from petisco.frameworks.flask.flask_extension_is_installed import (
    flask_extension_is_installed,
)
from petisco.fixtures import *
from .fixtures import *
from tests.modules.event.fixtures import *
from tests.modules.unit.controller.fixtures import *

if "PETISCO_END2END_TESTS" in os.environ:
    from tests.end2end.fixtures import *


if flask_extension_is_installed():
    pass



from meiga import Result

from petisco import controller_handler, TokenManager, InfoId
from tests.end2end.flask_app.user_name.application.get_user_name import GetUserName


def success_handler(result: Result):
    return {"name": result.value.value}, 200


@controller_handler(
    success_handler=success_handler,
    token_manager=TokenManager(token_type="USER_TOKEN", require_user=True),
)
def get_user_name(info_id: InfoId):
    return GetUserName.build().execute(info_id)



from typing import NamedTuple

import pytest
from meiga.assertions import assert_failure, assert_success

from petisco import UseCase, use_case_handler, ERROR, DEBUG, UnknownError
from meiga import Success, Failure, isFailure, isSuccess, Error

from tests.modules.unit.mocks.fake_logger import FakeLogger
from tests.modules.unit.mocks.log_message_mother import LogMessageMother


@pytest.mark.unit
def test_should_log_successfully_a_non_error_use_case_without_input_parameters_and_returning_a_string_result(
    given_any_petisco
):

    logger = FakeLogger()

    @use_case_handler(logger=logger)
    class MyUseCase(UseCase):
        def execute(self):
            return Success("Hello Petisco")

    MyUseCase().execute()

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Hello Petisco"
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_log_successfully_a_non_error_use_case_with_input_parameters_but_not_in_the_whitelist(
    given_any_petisco
):

    logger = FakeLogger()

    @use_case_handler(logger=logger)
    class MyUseCase(UseCase):
        def execute(self, client_id: str, user_id: str):
            return Success("Hello Petisco")

    MyUseCase().execute(client_id="client_id", user_id="user_id")

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Hello Petisco"
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_log_successfully_a_non_error_use_case_with_input_parameters(
    given_any_petisco
):

    logger = FakeLogger()

    @use_case_handler(
        logger=logger, logging_parameters_whitelist=["client_id", "user_id"]
    )
    class MyUseCase(UseCase):
        def execute(self, client_id: str, user_id: str):
            return Success("Hello Petisco")

    result = MyUseCase().execute(client_id="client_id", user_id="user_id")

    assert_success(result)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]
    third_logging_message = logger.get_logging_messages()[2]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase",
            message={"client_id": "client_id", "user_id": "user_id"},
        ).to_dict(),
    )
    assert third_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Hello Petisco"
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_log_successfully_a_filtered_object_by_blacklist_with_python_type_bytes(
    given_any_petisco
):

    logger = FakeLogger()

    @use_case_handler(logger=logger, logging_types_blacklist=[bytes])
    class MyUseCase(UseCase):
        def execute(self):
            return Success(b"This are bytes")

    result = MyUseCase().execute()

    assert_success(result)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Success result of type: bytes"
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_log_successfully_a_filtered_object_by_blacklist_with_own_named_tuple(
    given_any_petisco
):

    logger = FakeLogger()

    class BinaryInfo(NamedTuple):
        name: str
        data: bytes

    @use_case_handler(logger=logger, logging_types_blacklist=[BinaryInfo])
    class MyUseCase(UseCase):
        def execute(self):
            binary_info = BinaryInfo(name="my_data", data=b"This are bytes")
            return Success(binary_info)

    result = MyUseCase().execute()

    assert_success(result)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Success result of type: BinaryInfo"
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_log_successfully_a_filtered_object_by_blacklist_with_a_tuple(
    given_any_petisco
):

    logger = FakeLogger()

    @use_case_handler(logger=logger, logging_types_blacklist=[tuple])
    class MyUseCase(UseCase):
        def execute(self):
            binary_info = ("my_data", b"This are bytes")
            return Success(binary_info)

    result = MyUseCase().execute()

    assert_success(result)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Success result of type: tuple"
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_log_successfully_a_large_type_with_its_repr(given_any_petisco):

    logger = FakeLogger()

    class BinaryInfo(NamedTuple):
        name: str
        data: bytes

        def __repr__(self) -> str:
            return f"<BinaryInfo {self.name}, len(data)={len(self.data)}>"

    @use_case_handler(logger=logger)
    class MyUseCase(UseCase):
        def execute(self):
            binary_info = BinaryInfo(name="my_data", data=b"This are bytes")
            return Success(binary_info)

    result = MyUseCase().execute()

    assert_success(result)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="<BinaryInfo my_data, len(data)=14>"
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_log_successfully_an_error_returned_on_a_use_case(given_any_petisco):

    logger = FakeLogger()

    @use_case_handler(logger=logger)
    class MyUseCase(UseCase):
        def execute(self):
            return isFailure

    result = MyUseCase().execute()

    assert_failure(result, value_is_instance_of=Error)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        ERROR,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Result[status: failure | value: Error] "
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_log_successfully_an_error_raised_by_a_meiga_handler(given_any_petisco):

    logger = FakeLogger()

    class UserNotFoundError(Error):
        pass

    @use_case_handler(logger=logger)
    class MyUseCase(UseCase):
        def execute(self):
            Failure(UserNotFoundError()).unwrap_or_return()
            return isSuccess

    result = MyUseCase().execute()

    assert_failure(result, value_is_instance_of=UserNotFoundError)

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert second_logging_message == (
        ERROR,
        LogMessageMother.get_use_case(
            operation="MyUseCase",
            message="Result[status: failure | value: UserNotFoundError] ",
        ).to_dict(),
    )


@pytest.mark.unit
def test_should_use_case_handler_return_a_failure_with_unknown_error_when_raise_an_uncontrolled_exception(
    given_any_petisco
):

    logger = FakeLogger()
    expected_raised_exception = RuntimeError("uncontrolled exception")

    @use_case_handler(logger=logger)
    class MyUseCase(UseCase):
        def execute(self):
            raise expected_raised_exception

    result = MyUseCase().execute()

    assert_failure(
        result,
        value_is_instance_of=UnknownError,
        value_is_equal_to=UnknownError(expected_raised_exception),
    )

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert (
        "value: UnknownError (MyUseCase (Use Case)): RuntimeError: uncontrolled exception.\nTraceback (most recent call last):\n  File"
        in second_logging_message[1]["data"]["message"]
    )


@pytest.mark.unit
def test_should_use_case_handler_return_a_failure_with_unknown_error_when_raise_an_uncontrolled_exception_with_input_parameters(
    given_any_petisco
):

    logger = FakeLogger()
    expected_raised_exception = RuntimeError("uncontrolled exception")
    input_text = "my-text"
    input_verbose = True

    @use_case_handler(logger=logger)
    class MyUseCase(UseCase):
        def execute(self, text: str, verbose: bool):
            raise expected_raised_exception

    result = MyUseCase().execute(text=input_text, verbose=input_verbose)

    assert_failure(
        result,
        value_is_instance_of=UnknownError,
        value_is_equal_to=UnknownError(expected_raised_exception),
    )

    assert {"text": "my-text", "verbose": True} == result.value.input_parameters

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert (
        "value: UnknownError (MyUseCase (Use Case)): RuntimeError: uncontrolled exception.\nTraceback (most recent call last):\n  File"
        in second_logging_message[1]["data"]["message"]
    )


@pytest.mark.unit
def test_should_use_case_handler_return_a_failure_with_unknown_error_when_raise_an_uncontrolled_exception_with_non_defined_input_parameters(
    given_any_petisco
):

    logger = FakeLogger()
    expected_raised_exception = RuntimeError("uncontrolled exception")
    input_text = "my-text"
    input_verbose = True

    @use_case_handler(logger=logger)
    class MyUseCase(UseCase):
        def execute(self, text: str, verbose: bool):
            raise expected_raised_exception

    result = MyUseCase().execute(input_text, input_verbose)

    assert_failure(
        result,
        value_is_instance_of=UnknownError,
        value_is_equal_to=UnknownError(expected_raised_exception),
    )

    assert {"param_1": "my-text", "param_2": True} == result.value.input_parameters

    first_logging_message = logger.get_logging_messages()[0]
    second_logging_message = logger.get_logging_messages()[1]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_use_case(
            operation="MyUseCase", message="Running Use Case"
        ).to_dict(),
    )
    assert (
        "value: UnknownError (MyUseCase (Use Case)): RuntimeError: uncontrolled exception.\nTraceback (most recent call last):\n  File"
        in second_logging_message[1]["data"]["message"]
    )


@pytest.mark.unit
def test_should_log_successfully_a_non_error_use_case_with_default_parameters(
    given_any_petisco
):
    @use_case_handler()
    class MyUseCase(UseCase):
        def execute(self):
            return Success("Hello Petisco")

    MyUseCase().execute()



from typing import Dict, List, Callable
from abc import ABCMeta, abstractmethod

from petisco.event.shared.domain.event_subscriber import EventSubscriber


class IEventConsumer:

    __metaclass__ = ABCMeta

    def __repr__(self):
        return f"IEventConsumer"

    @classmethod
    def info(cls) -> Dict:
        return {"name": cls.__name__}

    @abstractmethod
    def add_subscribers(self, subscribers: List[EventSubscriber]):
        raise NotImplementedError

    @abstractmethod
    def add_subscriber_on_dead_letter(
        self, subscriber: EventSubscriber, handler: Callable
    ):
        raise NotImplementedError

    @abstractmethod
    def add_handler_on_store(self, handler: Callable):
        raise NotImplementedError

    @abstractmethod
    def add_handler_on_queue(self, queue_name: str, handler: Callable):
        raise NotImplementedError

    @abstractmethod
    def start(self):
        raise NotImplementedError

    @abstractmethod
    def stop(self):
        raise NotImplementedError



from typing import Dict

from meiga import Result, Error, Failure, Success

from petisco.application.petisco import Petisco
from petisco.use_case.use_case import UseCase
from petisco.modules.healthcheck.domain.persistence_error import PersistenceError


class HealthcheckProvider(UseCase):
    def execute(self, petisco: Petisco) -> Result[Dict, Error]:

        healthcheck = {
            "app_name": petisco._app_name,
            "app_version": petisco._app_version,
        }

        if petisco.persistence_sources:
            from petisco.persistence.sqlalchemy.sqlalchemy_session_scope import (
                session_scope,
            )

            for (
                persistence_source_name,
                persistence_source,
            ) in petisco.persistence_sources.items():
                if persistence_source["type"] == "sql":
                    with session_scope(persistence_source_name) as session:
                        try:
                            session.execute("SELECT 1")
                        except Exception as e:
                            return Failure(PersistenceError(str(e)))
            healthcheck["persistence_available"] = True

        return Success(healthcheck)



import threading
import traceback
from time import sleep
from typing import Callable, List, Optional

from pika import BasicProperties
from pika.adapters.blocking_connection import BlockingChannel
from pika.spec import Basic

from petisco.logger.interface_logger import ILogger
from petisco.logger.not_implemented_logger import NotImplementedLogger

from petisco.event.chaos.domain.interface_event_chaos import IEventChaos
from petisco.event.chaos.infrastructure.not_implemented_event_chaos import (
    NotImplementedEventChaos,
)

from petisco.event.consumer.domain.consumer_derived_action import ConsumerDerivedAction
from petisco.event.consumer.infrastructure.rabbitmq_event_consumer_logger import (
    RabbitMqEventConsumerLogger,
)
from petisco.event.consumer.infrastructure.rabbitmq_event_consumer_printer import (
    RabbitMqEventConsumerPrinter,
)
from petisco.event.consumer.infrastructure.rabbitmq_event_consumer_return_error import (
    RabbitMqEventConsumerReturnError,
)
from petisco.event.shared.domain.event import Event
from petisco.event.shared.domain.event_subscriber import EventSubscriber
from petisco.event.consumer.domain.interface_event_consumer import IEventConsumer
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_exchange_name_formatter import (
    RabbitMqExchangeNameFormatter,
)
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_event_subscriber_queue_name_formatter import (
    RabbitMqEventSubscriberQueueNameFormatter,
)
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_connector import (
    RabbitMqConnector,
)


class RabbitMqEventConsumer(IEventConsumer):
    def __init__(
        self,
        connector: RabbitMqConnector,
        organization: str,
        service: str,
        max_retries: int,
        verbose: bool = False,
        chaos: IEventChaos = NotImplementedEventChaos(),
        logger: Optional[ILogger] = NotImplementedLogger(),
    ):
        self.connector = connector
        self.exchange_name = f"{organization}.{service}"
        self.rabbitmq_key = f"consumer-{self.exchange_name}"
        self._fallback_store_exchange_name = f"retry.{organization}.store"
        self.max_retries = max_retries
        self._channel = self.connector.get_channel(self.rabbitmq_key)
        self.printer = RabbitMqEventConsumerPrinter(verbose)
        self.consumer_logger = RabbitMqEventConsumerLogger(logger)
        self.chaos = chaos

    def start(self):
        if not self._channel:
            raise RuntimeError(
                "RabbitMqEventConsumer: cannot start consuming event without any subscriber defined."
            )

        self._thread = threading.Thread(target=self._start)
        self._thread.start()

    def _start(self):
        self._channel.start_consuming()

    def add_subscribers(self, subscribers: List[EventSubscriber]):
        for subscriber in subscribers:
            queue_name = RabbitMqEventSubscriberQueueNameFormatter.format(
                subscriber, exchange_name=self.exchange_name
            )
            for handler in subscriber.handlers:
                self._channel.basic_consume(
                    queue=f"{queue_name}.{handler.__name__}",
                    on_message_callback=self.consumer(handler),
                )

    def add_subscriber_on_dead_letter(
        self, subscriber: EventSubscriber, handler: Callable
    ):
        queue_name = RabbitMqEventSubscriberQueueNameFormatter.format_dead_letter(
            subscriber, exchange_name=self.exchange_name
        )
        for handler_name in subscriber.get_handlers_names():
            self._channel.basic_consume(
                queue=f"{queue_name}.{handler_name}",
                on_message_callback=self.consumer(handler),
            )

    def add_handler_on_store(self, handler: Callable):
        is_store = True
        self._channel.basic_consume(
            queue="store", on_message_callback=self.consumer(handler, is_store)
        )

    def add_handler_on_queue(self, queue_name: str, handler: Callable):
        self._channel.basic_consume(
            queue=queue_name, on_message_callback=self.consumer(handler)
        )

    def consumer(self, handler: Callable, is_store: bool = False) -> Callable:
        def rabbitmq_consumer(
            ch: BlockingChannel,
            method: Basic.Deliver,
            properties: BasicProperties,
            body: bytes,
        ):
            self.printer.print_received_message(method, properties, body)

            if self.chaos.nack_simulation(ch, method):
                self.consumer_logger.log_nack_simulation(
                    method, properties, body, handler
                )
                return

            try:
                event = Event.from_json(body)
            except TypeError:
                event = Event.from_deprecated_json(body)
            except Exception as e:
                self.consumer_logger.log_parser_error(
                    method, properties, body, handler, e
                )
                ch.basic_nack(delivery_tag=method.delivery_tag)
                return

            self.chaos.delay()

            result = handler(event)
            result = self.chaos.simulate_failure_on_result(result)

            self.printer.print_context(handler, result)

            if result is None:
                raise RabbitMqEventConsumerReturnError(handler)

            derived_action = ConsumerDerivedAction()
            if result.is_failure:
                if not properties.headers:
                    properties.headers = {
                        "queue": f"{method.routing_key}.{handler.__name__}"
                    }
                derived_action = self.handle_consumption_error(
                    ch, method, properties, body, is_store
                )
            else:
                ch.basic_ack(delivery_tag=method.delivery_tag)

            self.consumer_logger.log(
                method, properties, body, handler, result, derived_action
            )
            self.printer.print_separator()

        return rabbitmq_consumer

    def handle_consumption_error(
        self,
        ch: BlockingChannel,
        method: Basic.Deliver,
        properties: BasicProperties,
        body: bytes,
        is_store: bool,
    ):

        if self.has_been_redelivered_too_much(properties):
            derived_action = self.send_to_dead_letter(ch, method, properties, body)
        else:
            derived_action = self.send_to_retry(ch, method, properties, body, is_store)

        ch.basic_ack(delivery_tag=method.delivery_tag)

        return derived_action

    def has_been_redelivered_too_much(self, properties: BasicProperties):
        if not properties.headers or "redelivery_count" not in properties.headers:
            if self.max_retries < 1:
                return True
            return False
        else:
            return properties.headers.get("redelivery_count") >= self.max_retries

    def _get_routing_key(self, routing_key: str, prefix: str):
        if routing_key.startswith("retry."):
            routing_key = routing_key.replace("retry.", prefix, 1)
        elif routing_key.startswith("dead_letter."):
            routing_key = routing_key.replace("dead_letter.", prefix, 1)
        else:
            routing_key = f"{prefix}{routing_key}"
        return routing_key

    def send_to_retry(
        self,
        ch: BlockingChannel,
        method: Basic.Deliver,
        properties: BasicProperties,
        body: bytes,
        is_store: bool = False,
    ):
        self.printer.print_action("send_to_retry")
        exchange_name = RabbitMqExchangeNameFormatter.retry(self.exchange_name)

        routing_key = method.routing_key
        if properties.headers:
            routing_key = properties.headers.get("queue", routing_key)

        if is_store:
            routing_key = "store"
            exchange_name = self._fallback_store_exchange_name

        routing_key = self._get_routing_key(routing_key, "retry.")
        updated_headers = self.send_message_to(
            exchange_name, ch, routing_key, properties, body
        )
        return ConsumerDerivedAction(
            action="send_to_retry",
            exchange_name=exchange_name,
            routing_key=routing_key,
            headers=updated_headers,
        )

    def send_to_dead_letter(
        self,
        ch: BlockingChannel,
        method: Basic.Deliver,
        properties: BasicProperties,
        body: bytes,
    ):
        self.printer.print_action("send_to_dead_letter")
        exchange_name = RabbitMqExchangeNameFormatter.dead_letter(self.exchange_name)
        routing_key = self._get_routing_key(method.routing_key, "dead_letter.")
        updated_headers = self.send_message_to(
            exchange_name, ch, routing_key, properties, body
        )
        return ConsumerDerivedAction(
            action="send_to_dead_letter",
            exchange_name=exchange_name,
            routing_key=routing_key,
            headers=updated_headers,
        )

    def send_message_to(
        self,
        exchange_name: str,
        ch: BlockingChannel,
        routing_key: str,
        properties: BasicProperties,
        body: bytes,
    ):
        if properties.headers:
            redelivery_count = properties.headers.get("redelivery_count", 0)
            properties.headers["redelivery_count"] = redelivery_count + 1
        else:
            properties.headers = {"redelivery_count": 1}

        self.printer.print_send_message_to(
            exchange_name, routing_key, properties.headers
        )

        ch.basic_publish(
            exchange=exchange_name,
            routing_key=routing_key,
            body=body,
            properties=properties,
        )

        return properties.headers

    def stop(self):
        def _log_stop_exception(e: Exception):
            from petisco import LogMessage, ERROR, Petisco

            logger = Petisco.get_logger()
            log_message = LogMessage(
                layer="petisco", operation=f"RabbitMQEventSubscriber"
            )
            message = f"Error stopping RabbitMQEventSubscriber: {repr(e.__class__)} {e} | {traceback.format_exc()}"
            logger.log(ERROR, log_message.set_message(message))

        if self._thread and self._thread.is_alive():
            self._unsubscribe_all()
            try:
                self._thread.join()
                self._thread = None
            except Exception as e:
                _log_stop_exception(e)

    def _unsubscribe_all(self):
        def _stop_consuming_consumer_channel():
            try:
                self._channel.stop_consuming()
                self._channel.cancel()
            except ValueError:
                pass

        def _await_for_stop_consuming_consumer_channel():
            sleep(2.0)

        self.connector.get_connection(self.rabbitmq_key).call_later(
            0, _stop_consuming_consumer_channel
        )

        _await_for_stop_consuming_consumer_channel()



from petisco import RabbitMqConnector

from petisco.logger.interface_logger import ILogger
from petisco.event.chaos.domain.interface_event_chaos import IEventChaos
from petisco.event.consumer.infrastructure.rabbitmq_event_consumer import (
    RabbitMqEventConsumer,
)
from petisco.logger.not_implemented_logger import NotImplementedLogger
from tests.modules.event.mothers.defaults import (
    DEFAULT_ORGANIZATION,
    DEFAULT_SERVICE,
    DEFAULT_MAX_RETRIES,
    DEFAULT_VERBOSE,
)


class RabbitMqEventConsumerMother:
    @staticmethod
    def default(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConsumer(
            connector,
            DEFAULT_ORGANIZATION,
            DEFAULT_SERVICE,
            DEFAULT_MAX_RETRIES,
            DEFAULT_VERBOSE,
        )

    @staticmethod
    def with_service(service: str, connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConsumer(
            connector,
            DEFAULT_ORGANIZATION,
            service,
            DEFAULT_MAX_RETRIES,
            DEFAULT_VERBOSE,
        )

    @staticmethod
    def with_max_retries(max_retries: int, connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConsumer(
            connector,
            DEFAULT_ORGANIZATION,
            DEFAULT_SERVICE,
            max_retries,
            DEFAULT_VERBOSE,
        )

    @staticmethod
    def without_retry(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConsumer(
            connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE, 0, DEFAULT_VERBOSE
        )

    @staticmethod
    def with_chaos(
        chaos: IEventChaos,
        max_retries: int,
        logger: ILogger = NotImplementedLogger(),
        connector: RabbitMqConnector = None,
    ):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConsumer(
            connector,
            DEFAULT_ORGANIZATION,
            DEFAULT_SERVICE,
            max_retries,
            DEFAULT_VERBOSE,
            chaos,
            logger,
        )



import requests
from meiga import Result, Error, Failure, Success
from requests.exceptions import MissingSchema, ConnectionError, Timeout

from petisco.http.request_errors import (
    MissingSchemaRequestError,
    TimeoutRequestError,
    ConnectionRequestError,
    UnknownRequestError,
)
from petisco.http.response import Response


class Request:
    @staticmethod
    def __is_binary_content(content_type):
        binary_types = ["binary", "video", "image", "multipart/form-data"]
        if any(binary_type in content_type for binary_type in binary_types):
            return True
        return False

    @staticmethod
    def get(
        url: str,
        binary_info: dict = None,
        string_info: dict = None,
        json_info: dict = None,
        headers=None,
    ) -> Result[Response, Error]:
        return Request.execute(url, "GET", binary_info, string_info, json_info, headers)

    @staticmethod
    def post(
        url: str,
        binary_info: dict = None,
        string_info: dict = None,
        json_info: dict = None,
        headers=None,
    ) -> Result[Response, Error]:
        return Request.execute(
            url, "POST", binary_info, string_info, json_info, headers
        )

    @staticmethod
    def patch(
        url: str,
        binary_info: dict = None,
        string_info: dict = None,
        json_info: dict = None,
        headers=None,
    ) -> Result[Response, Error]:
        return Request.execute(
            url, "PATCH", binary_info, string_info, json_info, headers
        )

    @staticmethod
    def delete(
        url: str,
        binary_info: dict = None,
        string_info: dict = None,
        json_info: dict = None,
        headers=None,
    ) -> Result[Response, Error]:
        return Request.execute(
            url, "DELETE", binary_info, string_info, json_info, headers
        )

    @staticmethod
    def execute(
        url: str,
        request: str = "POST",
        binary_info: dict = None,
        string_info: dict = None,
        json_info: dict = None,
        headers=None,
    ) -> Result[Response, Error]:
        response = None
        try:
            if request == "GET":
                response = requests.get(
                    url=url, params=string_info, headers=headers, timeout=(5, 25)
                )
            elif request == "POST":
                response = requests.post(
                    url=url,
                    files=binary_info,
                    data=string_info,
                    json=json_info,
                    headers=headers,
                    timeout=(5, 25),
                )
            elif request == "PATCH":
                response = requests.patch(
                    url=url,
                    data=string_info,
                    json=json_info,
                    headers=headers,
                    timeout=(5, 25),
                )
            elif request == "DELETE":
                response = requests.delete(url=url, headers=headers, timeout=(5, 25))

        except MissingSchema:
            return Failure(MissingSchemaRequestError())
        except Timeout:
            return Failure(TimeoutRequestError())
        except ConnectionError:
            return Failure(ConnectionRequestError())
        except Exception as e:
            return Failure(UnknownRequestError(error_message=e))

        if isinstance(response, requests.models.Response):
            if Request.__is_binary_content(response.headers.get("Content-Type", [])):
                return Success(
                    Response(
                        status_code=response.status_code,
                        content=response.content,
                        headers=response.headers,
                    )
                )
            try:
                json_response = response.json()
            except ValueError:
                json_response = response.text
            return Success(
                Response(
                    status_code=response.status_code,
                    content=json_response,
                    headers=response.headers,
                )
            )
        else:
            return Failure(UnknownRequestError())



from typing import Any

from petisco.domain.aggregate_roots.info_id import InfoId
from petisco.logger.log_message import LogMessage


class LogMessageMother:
    @staticmethod
    def get_controller(
        operation: str = None, info_id: InfoId = None, message: str = None
    ) -> LogMessage:
        return LogMessage(
            layer="controller",
            operation=operation,
            info_id=info_id,
            data={"message": message},
        )

    @staticmethod
    def get_use_case(
        operation: str = None, info_id: InfoId = None, message: Any = None
    ) -> LogMessage:
        return LogMessage(
            layer="use_case",
            operation=operation,
            info_id=info_id,
            data={"message": message},
        )

    @staticmethod
    def get_command(
        operation: str = None, info_id: InfoId = None, message: str = None
    ) -> LogMessage:
        return LogMessage(
            layer="command",
            operation=operation,
            info_id=info_id,
            data={"message": message},
        )

    @staticmethod
    def get_subscriber(operation: str = None, message: str = None) -> LogMessage:
        return LogMessage(
            layer="subscriber", operation=operation, data={"message": message}
        )

    @staticmethod
    def get_event_handler(operation: str = None, message: str = None) -> LogMessage:
        return LogMessage(
            layer="event_handler", operation=operation, data={"message": message}
        )



from typing import Dict, Optional

from dataclasses import dataclass
from petisco.domain.value_objects.client_id import ClientId
from petisco.domain.value_objects.user_id import UserId
from petisco.domain.value_objects.correlation_id import CorrelationId
from petisco.domain.aggregate_roots.aggregate_root import AggregateRoot
from petisco.security.token_decoder.token import Token


@dataclass
class InfoId(AggregateRoot):
    client_id: Optional[ClientId] = None
    user_id: Optional[UserId] = None
    correlation_id: Optional[CorrelationId] = None
    ip: Optional[str] = None

    def __init__(
        self,
        client_id: ClientId = None,
        user_id: UserId = None,
        correlation_id: CorrelationId = None,
        ip: str = None,
    ):
        self.client_id = client_id
        self.user_id = user_id
        self.correlation_id = (
            correlation_id if correlation_id else CorrelationId.generate()
        )
        self.ip = ip
        super().__init__()

    def __repr__(self):
        return f"[InfoId: [client_id: {self.client_id} | user_id: {self.user_id} | correlation_id: {self.correlation_id} | ip: {self.ip}]]"

    def __eq__(self, other):
        return (
            self.__class__ == other.__class__
            and self.client_id == other.client_id
            and self.user_id == other.user_id
            and self.correlation_id == other.correlation_id
            and self.ip == other.ip
        )

    @staticmethod
    def from_dict(kdict: dict):
        client_id = ClientId(kdict.get("client_id")) if kdict.get("client_id") else None
        user_id = ClientId(kdict.get("user_id")) if kdict.get("user_id") else None
        correlation_id = (
            ClientId(kdict.get("correlation_id"))
            if kdict.get("correlation_id")
            else None
        )
        ip = kdict.get("ip")

        return InfoId(
            client_id=client_id, user_id=user_id, correlation_id=correlation_id, ip=ip
        )

    def to_dict(self):
        return {
            "client_id": self.client_id.value if self.client_id else None,
            "user_id": self.user_id.value if self.user_id else None,
            "correlation_id": self.correlation_id.value
            if self.correlation_id
            else None,
            "ip": self.ip,
        }

    def update_from_headers(self, headers: Dict[str, str]):
        if headers:
            client_id = headers.get("X-Onboarding-Clientid")
            user_id = headers.get("X-Onboarding-Userid")
            correlation_id = headers.get("X-Correlation-Id")
            ip = headers.get("X-Forwarded-For")

            self.client_id = ClientId(client_id) if client_id else self.client_id
            self.user_id = UserId(user_id) if user_id else self.user_id
            self.correlation_id = (
                CorrelationId(correlation_id) if correlation_id else self.correlation_id
            )
            self.ip = ip if ip else self.ip
        return self

    @staticmethod
    def from_headers(headers: Dict[str, str]):
        if headers:
            info_id = InfoId.from_strings(
                headers.get("X-Onboarding-Clientid"),
                headers.get("X-Onboarding-Userid"),
                headers.get("X-Correlation-Id"),
                headers.get("X-Forwarded-For"),
            )
        else:
            info_id = InfoId()
        return info_id

    @staticmethod
    def from_token(token: Token):
        if token:
            info_id = InfoId.from_strings(
                token.client_id.value, token.user_id.value if token.user_id else None
            )
        else:
            info_id = InfoId()
        return info_id

    @staticmethod
    def from_strings(
        client_id: str = None,
        user_id: str = None,
        correlation_id: str = None,
        ip: str = None,
    ):
        return InfoId(
            ClientId(client_id) if client_id else None,
            UserId(user_id) if user_id else None,
            CorrelationId(correlation_id) if correlation_id else None,
            ip,
        )

    def get_http_headers(self):
        headers = {}

        if self.client_id:
            headers["X-Onboarding-Clientid"] = self.client_id.value
        if self.user_id:
            headers["X-Onboarding-Userid"] = self.user_id.value
        if self.correlation_id:
            headers["X-Correlation-Id"] = self.correlation_id.value

        return headers if len(headers) > 0 else None



import os

import pytest

from petisco.persistence.pymongo.mongodb_is_running_locally import (
    mongodb_is_running_locally,
)
from petisco.persistence.pymongo.pymongo_persistence import PyMongoPersistence
from petisco.persistence.pymongo.pymongo_persistence_config import (
    PyMongoPersistenceConfig,
)
from petisco.persistence.pymongo.pymongo_persistence_connector import (
    PyMongoPersistenceConnector,
)

username = os.getenv("MONGODB_USERNAME")
password = os.getenv("MONGODB_PASSWORD")
port = int(os.getenv("MONGODB_PORT"))
host = os.getenv("MONGODB_HOST")
database = os.getenv("MONGODB_DATABASE")


@pytest.mark.integration
@pytest.mark.skipif(
    not mongodb_is_running_locally(
        host=host, username=username, password=password, port=port
    ),
    reason="MongoDB is not running locally",
)
def test_should_initialize_persistence():
    config = PyMongoPersistenceConfig(
        user=username, password=password, port=port, host=host, database=database
    )
    connector = PyMongoPersistenceConnector(config=config)
    connector.execute()
    persistence = PyMongoPersistence.get_instance()
    persistence.client.server_info()
    persistence.client.close()



import os
from typing import Callable

from petisco import SqlAlchemyPersistenceConfig, SqlAlchemyPersistenceConnector


def config_persistence(import_database_models: Callable):
    config = SqlAlchemyPersistenceConfig(
        server=os.environ.get("SQL_SERVER"), database=os.environ.get("SQL_DATABASE")
    )
    persistence_connector = SqlAlchemyPersistenceConnector(
        config=config, name="petisco", import_database_models=import_database_models
    )
    persistence_connector.execute()



from petisco.domain.value_objects.user_id import UserId

DEFAULT_ORGANIZATION = "alice"
DEFAULT_SERVICE = "petisco"
DEFAULT_EXCHANGE_NAME = f"{DEFAULT_ORGANIZATION}.{DEFAULT_SERVICE}"
DEFAULT_USER_ID = UserId("userpwlxU3yGsird")
DEFAULT_MAX_RETRIES = 5
DEFAULT_VERBOSE = True
